# -*- coding: utf-8 -*-

# Copyright (C) 2012-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2012-2013 by Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>
# Copyright (C) 2012-2013 by Heinz-Markus Graesing <heinz-m.graesing@obviously-nice.de>
#
# X2Go Session Broker is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# X2Go Session Broker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

# modules
import os
import uuid
import getpass
import socket

import logging
from loggers import logger_broker, logger_access, logger_error, X2GOBROKER_DAEMON_USER

X2GOBROKER_USER =  getpass.getuser()

if os.environ.has_key('X2GOBROKER_DAEMON_GROUP'):
    X2GOBROKER_DAEMON_GROUP=os.environ['X2GOBROKER_DAEMON_GROUP']
else:
    X2GOBROKER_DAEMON_GROUP="x2gobroker"
if os.environ.has_key('X2GOBROKER_AGENT_USER'):
    X2GOBROKER_AGENT_USER=os.environ['X2GOBROKER_AGENT_USER']
else:
    X2GOBROKER_AGENT_USER="x2gobroker"

###
### dynamic default values, influencable through os.environ...
###

if os.environ.has_key('X2GOBROKER_DEBUG'):
    X2GOBROKER_DEBUG = ( os.environ['X2GOBROKER_DEBUG'].lower() in ('1', 'on', 'true', 'yes', ) )
else:
    X2GOBROKER_DEBUG = False
if os.environ.has_key('X2GOBROKER_TESTSUITE'):
    X2GOBROKER_TESTSUITE = ( os.environ['X2GOBROKER_TESTSUITE'].lower() in ('1', 'on', 'true', 'yes', ) )
else:
    X2GOBROKER_TESTSUITE = False

# enforce debugging for interactive usage
if X2GOBROKER_USER != X2GOBROKER_DAEMON_USER:
    X2GOBROKER_DEBUG = True

# raise log levels to CRITICAL if we are running the unittests...
if X2GOBROKER_TESTSUITE:
    logger_broker.setLevel(logging.CRITICAL)
    logger_access.setLevel(logging.CRITICAL)
    logger_error.setLevel(logging.CRITICAL)

if os.environ.has_key('X2GOBROKER_CONFIG'):
    X2GOBROKER_CONFIG = os.environ['X2GOBROKER_CONFIG']
else:
    X2GOBROKER_CONFIG = "/etc/x2go/x2gobroker.conf"

if os.environ.has_key('X2GOBROKER_SESSIONPROFILES'):
    X2GOBROKER_SESSIONPROFILES = os.environ['X2GOBROKER_SESSIONPROFILES']
else:
    X2GOBROKER_SESSIONPROFILES = "/etc/x2go/broker/x2gobroker-sessionprofiles.conf"

if os.environ.has_key('X2GOBROKER_AGENT_CMD'):
    X2GOBROKER_AGENT_CMD = os.environ['X2GOBROKER_AGENT_CMD']
else:
    X2GOBROKER_AGENT_CMD = "/usr/lib/x2go/x2gobroker-agent"

if os.environ.has_key('X2GOBROKER_AUTHSERVICE_SOCKET'):
    X2GOBROKER_AUTHSERVICE_SOCKET=os.environ['X2GOBROKER_AUTHSERVICE_SOCKET']
else:
    X2GOBROKER_AUTHSERVICE_SOCKET="/run/x2gobroker/x2gobroker-authservice.socket"

if os.environ.has_key('X2GOBROKER_DEFAULT_BACKEND'):
    X2GOBROKER_DEFAULT_BACKEND = os.environ['X2GOBROKER_DEFAULT_BACKEND']
else:
    X2GOBROKER_DEFAULT_BACKEND = "zeroconf"

if os.environ.has_key('X2GOBROKER_SSL_CERTFILE'):
    X2GOBROKER_SSL_CERTFILE = os.environ['X2GOBROKER_SSL_CERTFILE']
else:
    X2GOBROKER_SSL_CERTFILE = ""

if os.environ.has_key('X2GOBROKER_SSL_KEYFILE'):
    X2GOBROKER_SSL_KEYFILE = os.environ['X2GOBROKER_SSL_KEYFILE']
else:
    X2GOBROKER_SSL_KEYFILE = ""

###
### static / hard-coded defaults
###

if socket.gethostname().find('.') >= 0:
    X2GOBROKER_HOSTNAME = socket.gethostname()
else:
    X2GOBROKER_HOSTNAME = socket.gethostbyaddr(socket.gethostname())[0]

# the home directory of the user that the daemon/cgi runs as
X2GOBROKER_HOME = os.path.normpath(os.path.expanduser('~{broker_uid}'.format(broker_uid=X2GOBROKER_DAEMON_USER)))

# defaults for X2Go Sessino Broker configuration file
X2GOBROKER_CONFIG_DEFAULTS = {
    'global': {
        u'backend': u'zeroconf',
        u'check-credentials': True,
        u'require-cookie-auth': False,
        u'use-static-cookie': False,
        u'my-cookie': uuid.uuid4(),
        u'enable-plain-output': True,
        u'enable-uccs-output': False,
        u'my-uccs-url-base': 'http://localhost:8080/',
        u'enable-json-output': False,
        u'enable-html-output':  False,
        u'default-auth-mech': u'pam',
        u'default-user-db': u'libnss',
        u'default-group-db': u'libnss',
        u'ignore-primary-group-memberships': True,
        u'default-session-autologin': False,
        u'default-authorized-keys': u'%h/.x2go/authorized_keys',
        u'default-agent-query-mode': u'LOCAL',
    },
    'zeroconf': {
        u'enable': True,
        u'auth-mech': u'pam',
        u'user-db': u'libnss',
        u'group-db': u'libnss',
        u'desktop-shell': u'KDE',
    },
    'inifile': {
        u'enable': False,
        u'session-profiles': u'/etc/x2go/broker/x2gobroker-sessionprofiles.conf',
        u'auth-mech': u'',
        u'user-db': u'',
        u'group-db': u'',
    },
    'ldap': {
        u'enable': False,
        u'auth-mech': u'ldap',
        u'user-db': u'ldap',
        u'group-db': u'ldap',
        u'uri': u'ldap://localhost:389',
        u'base': u'dc=example,dc=org',
        u'user-search-filter': u'(&(objectClass=posixAccount)(uid=*))',
        u'host-search-filter': u'(&(objectClass=ipHost)(serial=X2GoServer)(cn=*))',
        u'group-search-filter': u'(&(objectClass=posifxGroup)(cn=*))',
        u'starttls': False,
        u'agent-query-mode': u'SSH',
    },
}

X2GO_DESKTOP_SESSIONS= [
    'KDE',
    'GNOME',
    'XFCE',
    'CINNAMON',
    'MATE',
    'XFCE',
    'LXDE',
    'TRINITY',
    'UNITY',
    'XDMCP',
]

# defaults for X2Go Sessino Broker session profiles file
X2GOBROKER_SESSIONPROFILE_DEFAULTS = {
    u'DEFAULT': {
        u'command': u'TERMINAL',
        u'defsndport': True,
        u'useiconv': False,
        u'iconvfrom': u'UTF-8',
        u'height': 600,
        u'export': u'',
        u'quality': 9,
        u'fullscreen': False,
        u'layout': u'',
        u'useexports': True,
        u'width': 800,
        u'speed': 2,
        u'soundsystem': u'pulse',
        u'print': True,
        u'type': u'auto',
        u'sndport': 4713,
        u'xinerama': True,
        u'variant': u'',
        u'usekbd': True,
        u'fstunnel': True,
        u'applications': [u'TERMINAL',u'WWWBROWSER',u'MAILCLIENT',u'OFFICE'],
        u'multidisp': False,
        u'sshproxyport': 22,
        u'sound': True,
        u'rootless': True,
        u'iconvto': u'UTF-8',
        u'soundtunnel': True,
        u'dpi': 96,
        u'sshport': 22,
        u'setdpi': 0,
        u'pack': u'16m-jpeg',
        u'user': '',
        u'host': [ u'localhost', ],
        u'directrdp': False,
        u'acl-users-allow': [],
        u'acl-users-deny': [],
        u'acl-users-order': '',
        u'acl-groups-allow': [],
        u'acl-groups-deny': [],
        u'acl-groups-order': '',
        u'acl-clients-allow': [],
        u'acl-clients-deny': [],
        u'acl-clients-order': '',
        u'acl-any-order': u'deny-allow',
    },
}
