#!/usr/bin/perl -XU

# This file is part of the  X2Go Project - http://www.x2go.org
# Copyright (C) 2011-2013 by Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>
# Copyright (C) 2011-2013 by Heinz-Markus Graesing <heinz-m.graesing@obviously-nice.de>
# Copyright (C) 2012-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
# X2Go Session Broker is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# X2Go Session Broker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

use strict;

use File::Basename;

sub InitX2GoUser
{
	my ($user, $uidNumber, $gidNumber, $home)=@_;

	if ( -f "/etc/x2go/x2gosql/sql" )
	{
		# if we use the PostgreSQL session db backend we may have to add the
		# user to the session database...
		open(F,"/etc/x2go/x2gosql/sql");
		my @buf = <F>;
		close(F);
		if ( grep (/^backend=sqlite.*/, @buf) ) {
			#if ( ! -e "$home/.x2go/sqlpass" )
			###
			### FIXME: make the below code robust if homes are on NFS
			###
			###
			#{
			#	open my $save_out, ">&STDOUT";
			#	close (STDOUT);
			#	system "x2godbadmin", "--adduser", $user;
			#	open STDOUT, ">&", $save_out;
			#}
		}
	}
}

sub AddAuthKey
{
	my ($uid, $uidNumber, $gidNumber, $home, $pubkey, $authkeyfile)=@_;

	# rewrite %%, %u and %h in authkeyfile string
	$authkeyfile =~ s/%u/$uid/;
	$authkeyfile =~ s/%h/$home/;
	$authkeyfile =~ s/%%/%/;

	my $authkeydir = dirname($authkeyfile);
	if ( ! $authkeyfile =~ m/\/.*/ )
	{
		$authkeyfile = "$home/$authkeyfile";
	}

	# make sure dir and file for authorized_keys do exist
	system ("su - $uid -c \"mkdir -p $authkeydir\"");
	system ("su - $uid -c \"touch $authkeyfile\"");
	my $authorized_keys = `su - $uid -c \"cat $authkeyfile\"`;
	if ( ! ( $authorized_keys =~ m/$pubkey/ ) )
	{
		system("su - $uid -c \"echo $pubkey >> $authkeyfile\"");
	}
}

sub DelAuthKey
{
	my ($uid, $uidNumber, $gidNumber, $home, $pubkey, $authkeyfile)=@_;

	# rewrite %%, %u and %h in authkeyfile string
	$authkeyfile =~ s/%u/$uid/;
	$authkeyfile =~ s/%h/$home/;
	$authkeyfile =~ s/%%/%/;
	if ( ! $authkeyfile =~ m/\/.*/ )
	{
		$authkeyfile = "$home/$authkeyfile";
	}
	system("su - $uid -c \"sed -e s\!'$pubkey'\!''\! -e '/^\$/d' -i $authkeyfile\" 1>/dev/null 2>/dev/null");
}

$< = $>;
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};
$ENV{'PATH'} = '/bin:/usr/bin';

my $username=shift or die;
my $mode=shift or die;

my  ($uid, $passwd, $uidNumber, $gidNumber, $quota, $comment, $gcos, $home, $shell, $expire) = getpwnam($username);

if($uidNumber < 1000)
{
	die 'operation on system user';
}

if($mode eq 'listsessions')
{
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	exec ("/bin/su - $uid -c \"x2golistsessions --all-servers\"");
}

if( ($mode eq 'findbusyservers_by_sessionstats') || ($mode eq 'findbusyservers'))
{

	# Normally the session broker setup knows about all servers,
	# make sure your configuration of X2Go Session Broker is correct and
	# lists all available servers.

	# The findbusyservers algorithm only returns servers that are currently
	# in use (i.e. have running or suspended sessions on them). So the
	# result may be empty or contain a server list not containing all
	# available servers.

	# The logic of findbusyservers is this, then:
	#   1. if no server is returned, any of the configured servers is best server
	#   2. if some servers are returned, a best server is one that is not returned
	#   3. if all configured servers are returned, than evaluate the usage value
	#      (e.g. 90:server1, 20:server2, 10:server3 -> best server is server3)

	# The above interpretation has to be handled by the broker implementation
	# calling »x2gobroker-agent findbusyservers«.

	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	my $busy_servers = `/bin/su - $uid -c \"x2gogetservers\"`;

	my %server_load = ();
	my $num_sessions = 0;
	foreach (split('\n', $busy_servers))
	{
		my ($hostname, $num_users) = split(' ', $_);
		$server_load{$hostname} = $num_users;
		$num_sessions += $num_users;
	}

	# render the output result
	my @result;
	for my $hostname ( keys %server_load ) {
		my $available = $server_load{$hostname}/$num_sessions*100;
		push @result, sprintf '%1$d:%2$s', $available, $hostname;
	}
	print join("\n", sort @result);
	print "\n";
}


if($mode eq 'getservers')
{
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	exec ("/bin/su - $uid -c \"x2gogetservers\"");
}

if($mode eq 'addauthkey')
{
	my $pubkey = shift or die;
	my $authkeyfile = shift or die;
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	AddAuthKey($uid, $uidNumber, $gidNumber, $home, $pubkey, $authkeyfile);
}

if($mode eq 'delauthkey')
{
	my $pubkey = shift or die;
	my $authkeyfile = shift or die;
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	DelAuthKey($uid, $uidNumber, $gidNumber, $home, $pubkey, $authkeyfile);
}

if($mode eq 'suspendsession')
{
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	my $sid=shift;
	exec ("/bin/su - $uid -c \"\$(x2gopath lib)/x2gochangestatus S $sid\"");
}

if($mode eq 'terminatesession')
{
	InitX2GoUser($uid, $uidNumber, $gidNumber, $home);
	print "OK\n";
	my $sid=shift;
	exec ("/bin/su - $uid -c \"\$(x2gopath lib)/x2gochangestatus T $sid\"");
}

if($mode eq 'ping')
{
	print "OK\n";
}
